; GPS-PROJEKT	JULI 2006													
; Bernhard.Erfurt@gmx.de													
; VERSION 2
;
; MAI 2019
; Dominik Kristen
;  																

; KURZBESCHREIBUNG:															
; Die NMEA-Datenstze $GGARMC $GPGGA und $GPGSV werden einzeln eingelesen
; und	
; die erforderlichen DATEN (Koordinaten, Hhe, Kurs, Geschwindigkeit)		
; herausgefiltert,															
; Anschlieend erfolgt eine Berechnung der aktuellen Position zu der 		
; vorher einprogrammierten Ziel-Position.									
;																			
; Das Display zeigt im Grundmenue folgende Daten an: 						
; - die Entfernung zum Ziel (m/km)											
; - die Hhe (absolut/Differenz)											
; - KURS																	
; - Geschwindigkeit 														
; - Anzahl der empfangenen Satelliten										
; - Betriebsspannung														
;																			

; Anmerkung																	
; Die Berechnung der Entfernung zum Ziel ist in dieser Version 				
; auf den RAUM Deutschland (51 GRAD-Nrdliche Breite) angepasst worden		
; Bei Bedarf kann der "FAKTOR_X" entsprechend angepasst werden				

; DATEIEN:									
; "GPS_ALLGEMEIN.asm" 						
; "GPS_LCD.asm" 							
; "GPS_FUNKTIONEN.asm" 						
; "GPS_INTERRUPT.asm" 						
; "GPS_BERECHNUNG.asm" 						
; "GPS_TEXTE.asm"
; "GPS_MENU.asm"											
; "GPS_EEPROM.asm"

;											
; PINBELEGUNG ATmega8						
; 											
; PB0 ==>	LCD D0							
; PB1 ==>	LCD D1							
; PB2 ==>	LCD D3							
; PB3 ==>	LCD D4							
; PB4 ==>	LCD RS							
; PB5 ==>	LCD Enable						
; PB6 ==>	Quarz							
; PB7 ==>	Quarz							
;											
; PC3 ==>	LCD-BELEUCHTUNG	 (LOW =ON)		
; 											
; PD0 ==>	EINGANG RXD						
; PD1 ==>		
; PD2 ==>	TASTER-1		 (ENTER)		
; PD3 ==>	TASTER-2		 (+)			
; PD4 ==>	TASTER-3		 (-)			
; PD5 ==>	TASTER-4		 (Escape)		




; ATmega8							
.include "m8def.inc"
; CPU								
.equ TAKT					= 16000000
; GPS								
.equ FAKTOR_KNOTEN			= 1852				; 1 kn = 1,852 km/h 
.equ FAKTOR_X				=  70				; Faktor fr Differenz stliche LNGE 	
.equ FAKTOR_Y				= 111				; Faktor fr Different Nrdliche Breite	
; TIMER								
.equ TIMERWERT				= 65536-6250		;(2 hoch 16 minus....) ==> 0,1s			
.equ EEPROM_AUTO_SPEICHERN	= 2					; Zeit in Minuten zum speichern der Betriebsstunden (1...59)		
; USART								
.equ BRR					= TAKT/(4800*16)-1	; 4800 BAUD	
; LEDS 								
.equ LED_C_LCD				= 3
; TASTEN							
.equ TASTE_NULL				= 0
.equ TASTE_ESC				= 1
.equ TASTE_MINUS			= 2
.equ TASTE_PLUS				= 3
.equ TASTE_ENTER			= 4
; LCD								
.equ LCD_PORT				= PORTB
.equ LCD_D0					= 0
.equ LCD_D1					= 1
.equ LCD_D2					= 2
.equ LCD_D3					= 3
.equ LCD_RS					= 4
.equ LCD_E					= 5
; EEPROM BELEGUNG					
.equ adr_EEPROM_BEREICH_SYSTEMEINSTELLUNGEN=0	; BEREICH SYSTEMEINSTELLUNGEN

.equ adr_EEPROM_REF_A_X_LAENGE_BYTE1= 101		; REFERENZ A (LNGE)	
.equ adr_EEPROM_REF_A_X_LAENGE_BYTE2= 102
.equ adr_EEPROM_REF_A_X_LAENGE_BYTE3= 103
.equ adr_EEPROM_REF_A_X_LAENGE_BYTE4= 104
.equ adr_EEPROM_REF_A_Y_BREITE_BYTE1= 105		; REFERENZ A (BREITE)	
.equ adr_EEPROM_REF_A_Y_BREITE_BYTE2= 106
.equ adr_EEPROM_REF_A_Y_BREITE_BYTE3= 107
.equ adr_EEPROM_REF_A_Y_BREITE_BYTE4= 108
.equ adr_EEPROM_REF_A_HOEHE_L		= 109		; REFERENZ A HHE		
.equ adr_EEPROM_REF_A_HOEHE_H		= 110


.equ adr_EEPROM_REF_B_X_LAENGE_BYTE1= 201		; REFERENZ B (LNGE)	
.equ adr_EEPROM_REF_B_X_LAENGE_BYTE2= 202
.equ adr_EEPROM_REF_B_X_LAENGE_BYTE3= 203
.equ adr_EEPROM_REF_B_X_LAENGE_BYTE4= 204
.equ adr_EEPROM_REF_B_Y_BREITE_BYTE1= 205		; REFERENZ B (BREITE)	
.equ adr_EEPROM_REF_B_Y_BREITE_BYTE2= 206
.equ adr_EEPROM_REF_B_Y_BREITE_BYTE3= 207
.equ adr_EEPROM_REF_B_Y_BREITE_BYTE4= 208
.equ adr_EEPROM_REF_B_HOEHE_L		= 209		; REFERENZ B HHE		
.equ adr_EEPROM_REF_B_HOEHE_H		= 210


.equ adr_EEPROM_REF_C_X_LAENGE_BYTE1= 301		; REFERENZ C (LNGE)	
.equ adr_EEPROM_REF_C_X_LAENGE_BYTE2= 302
.equ adr_EEPROM_REF_C_X_LAENGE_BYTE3= 303
.equ adr_EEPROM_REF_C_X_LAENGE_BYTE4= 304
.equ adr_EEPROM_REF_C_Y_BREITE_BYTE1= 305		; REFERENZ C (BREITE)	
.equ adr_EEPROM_REF_C_Y_BREITE_BYTE2= 306
.equ adr_EEPROM_REF_C_Y_BREITE_BYTE3= 307
.equ adr_EEPROM_REF_C_Y_BREITE_BYTE4= 308
.equ adr_EEPROM_REF_C_HOEHE_L		= 309		; REFERENZ C HHE		
.equ adr_EEPROM_REF_C_HOEHE_H		= 310


; SRAM-Belegung																			
.equ adr_DATEN						= 0x0060+0	; START Adresse aufbereiteter Daten		
.equ adr_SYSTEM_ss10				= 0x0060+0	; 10-tel Sekunde						
.equ adr_BETRIEBSSTUNDEN_ss			= 0x0060+1	; Betriebsstunden	
.equ adr_BETRIEBSSTUNDEN_mm			= 0x0060+2	; Betriebsstunden	
.equ adr_BETRIEBSSTUNDEN_hh			= 0x0060+3	; Betriebsstunden	
.equ adr_BETRIEBSSTUNDEN_SPEICHERN	= 0x0060+4	; 1=Betriebsstunden speichern		

.equ adr_ANZ_PROGRAMMSTARTS_L		= 0x0060+8			; ANZAHL PROGRAMMSTARTS L	
.equ adr_ANZ_PROGRAMMSTARTS_H		= 0x0060+9			; ANZAHL PROGRAMMSTARTS H	

.equ adr_BACKLIGHT					= 0x0060+10			; 	
.equ adr_ADC_BYTE_2 	 			= 0x0060+11			; BYTE2 fr ADC MITTELWERTBILDUNG	
.equ adr_ADC_BYTE_3 				= 0x0060+12			; BYTE3 fr ADC MITTELWERTBILDUNG	
.equ adr_TEMPERATUR_L				= 0x0060+13			; Temperatur						
.equ adr_TEMPERATUR_H				= 0x0060+14
.equ adr_QTH_LON					= 0x0060+15			; SPANNUNG							
.equ adr_QTH_LAT					= 0x0060+16
.equ adr_UNIT						= 0x0060+17			;   	


.equ adr_GPS_ONLINE_hh				= 0x0060+20		; ZEIT-GPS-ONLINE					
.equ adr_GPS_ONLINE_mm				= 0x0060+21
.equ adr_GPS_ONLINE_ss				= 0x0060+22
.equ adr_GPS_OFFLINE_hh				= 0x0060+23		; ZEIT-GPS-OFFLINE					
.equ adr_GPS_OFFLINE_mm				= 0x0060+24
.equ adr_GPS_OFFLINE_ss				= 0x0060+25

.equ adr_GPS_TAG					= 0x0060+30		; UTC		
.equ adr_GPS_MONAT					= 0x0060+31
.equ adr_GPS_JAHR					= 0x0060+32
.equ adr_GPS_hh						= 0x0060+33		; UTC		
.equ adr_GPS_mm						= 0x0060+34
.equ adr_GPS_ss						= 0x0060+35

.equ adr_GPS_HOEHE_L				= 0x0060+40		; GPS HHE	
.equ adr_GPS_HOEHE_H				= 0x0060+41
.equ adr_GPS_HOEHE_OFFSET_L			= 0x0060+42		; HHE-OFFSET
.equ adr_GPS_HOEHE_OFFSET_H			= 0x0060+43
.equ adr_GPS_HOEHE_BERECHNET_L		= 0x0060+44		; HHE-ERGEBNIS
.equ adr_GPS_HOEHE_BERECHNET_H		= 0x0060+45

.equ adr_GPS_GESCHWINDIGKEIT_L		= 0x0060+50		; GPS SOG		
.equ adr_GPS_GESCHWINDIGKEIT_H		= 0x0060+51
.equ adr_GPS_KURS_L					= 0x0060+52		; GPS KURS		
.equ adr_GPS_KURS_H					= 0x0060+53
.equ adr_GPS_ANZ_SAT_GESAMT			= 0x0060+54		; GPS SATELLITEN
.equ adr_GPS_ANZ_SAT_EMPFANGEN		= 0x0060+55		; GPS SATELLITEN

.equ adr_GPS_AKTIV					= 0x0060+58		; A / V 		

.equ adr_GPS_X_LAENGE_BYTE1			= 0x0060+60		; GPS POSITION-BREITE (dezimal)
.equ adr_GPS_X_LAENGE_BYTE2			= 0x0060+61
.equ adr_GPS_X_LAENGE_BYTE3			= 0x0060+62
.equ adr_GPS_X_LAENGE_BYTE4			= 0x0060+63

.equ adr_GPS_Y_BREITE_BYTE1			= 0x0060+64		; GPS POSITION-LNGE (dezimal)
.equ adr_GPS_Y_BREITE_BYTE2			= 0x0060+65
.equ adr_GPS_Y_BREITE_BYTE3			= 0x0060+66
.equ adr_GPS_Y_BREITE_BYTE4			= 0x0060+67

.equ adr_GPS_REFERENZ_X_LAENGE_BYTE1= 0x0060+70		; aktuelle REFERENZPOSITION	LNGE 
.equ adr_GPS_REFERENZ_X_LAENGE_BYTE2= 0x0060+71
.equ adr_GPS_REFERENZ_X_LAENGE_BYTE3= 0x0060+72
.equ adr_GPS_REFERENZ_X_LAENGE_BYTE4= 0x0060+73

.equ adr_GPS_REFERENZ_Y_BREITE_BYTE1= 0x0060+80		; aktuelle REFERENZPOSITION	BREITE
.equ adr_GPS_REFERENZ_Y_BREITE_BYTE2= 0x0060+81
.equ adr_GPS_REFERENZ_Y_BREITE_BYTE3= 0x0060+82
.equ adr_GPS_REFERENZ_Y_BREITE_BYTE4= 0x0060+83

.equ adr_GPS_REFERENZ_HOEHE_L		= 0x0060+85		; aktuelle REFERENZ HHE			
.equ adr_GPS_REFERENZ_HOEHE_H		= 0x0060+86

.equ adr_X_LAENGE_DIFFERENZ_BYTE1	= 0x0060+90	; Zwischenergebnis	
.equ adr_X_LAENGE_DIFFERENZ_BYTE2	= 0x0060+91
.equ adr_X_LAENGE_DIFFERENZ_BYTE3	= 0x0060+92
.equ adr_X_LAENGE_DIFFERENZ_BYTE4	= 0x0060+93

.equ adr_Y_BREITE_DIFFERENZ_BYTE1	= 0x0060+94	; Zwischenergebnis	
.equ adr_Y_BREITE_DIFFERENZ_BYTE2	= 0x0060+95
.equ adr_Y_BREITE_DIFFERENZ_BYTE3	= 0x0060+96
.equ adr_Y_BREITE_DIFFERENZ_BYTE4	= 0x0060+97

.equ adr_GPS_ENTFERNUNG_BYTE_1		= 0x0060+100		; BERECHNETE ENTFERNUNG zum ZIEL
.equ adr_GPS_ENTFERNUNG_BYTE_2		= 0x0060+101
.equ adr_GPS_ENTFERNUNG_BYTE_3		= 0x0060+102
.equ adr_GPS_ENTFERNUNG_BYTE_4		= 0x0060+103

.equ adr_GPS_ENTFERNUNG_in_km		= 0x0060+104		; MASSEINHEIT 0=m 1=km

.equ adr_GPS_PEGEL_1_1				= 0x0060+110	; PEGELWERTE aus $GPGSV,x,1	
.equ adr_GPS_PEGEL_1_2				= 0x0060+111
.equ adr_GPS_PEGEL_1_3				= 0x0060+112
.equ adr_GPS_PEGEL_1_4				= 0x0060+113

.equ adr_GPS_PEGEL_2_1				= 0x0060+114	; PEGELWERTE aus $GPGSV,x,2	
.equ adr_GPS_PEGEL_2_2				= 0x0060+115
.equ adr_GPS_PEGEL_2_3				= 0x0060+116
.equ adr_GPS_PEGEL_2_4				= 0x0060+117

.equ adr_GPS_PEGEL_3_1				= 0x0060+118	; PEGELWERTE aus $GPGSV,x,3	
.equ adr_GPS_PEGEL_3_2				= 0x0060+119
.equ adr_GPS_PEGEL_3_3				= 0x0060+120
.equ adr_GPS_PEGEL_3_4				= 0x0060+121

.equ adr_GPS_PEGEL_SUMME_L			= 0x0060+122	; SUMME aller PEGELWERTE
.equ adr_GPS_PEGEL_SUMME_H			= 0x0060+123
.equ adr_GPS_PEGEL_ANZAHL			= 0x0060+124	; ANZAHL der PEGELWERTE	


.equ adr_GPS_DATEN_SCHREIB_POSITION_L	= 0x0060+210	; DATEN im SRAM		
.equ adr_GPS_DATEN_SCHREIB_POSITION_H	= 0x0060+211

.equ adr_GPS_DATEN_ZEICHEN_ANZAHL		= 0x0060+220	; DATEN im SRAM			
.equ adr_GPS_DATEN_STRING_KOMPLETT		= 0x0060+221	; 1=STRING komplett eingelesen "GP....*"

.equ adr_GPS_DATEN_BEREICH_BEGINN		= 0x0060+250	; GPS-DATEN-STRING (max 255 Zeichen)	

.equ adr_GPS_GPRMC_KOPIE_BEGINN			= 0x0060+600	; KOPIE vom $GPRMC STRING (max 100 ZEICHEN)	


; REGISTER					
.def TASTE				=R2
.def MENUE_0			=R3
.def MENUE_1			=R4

.def REFERENZ_NUMMER	=R5			; A(EEPROM),B(EEPROM),C(EEPROM),1,2,3,4,5,6,7,8,9,10...

.def GPS_EMPFANGSZEICHEN=R9

.def temp  = r16
.def temp1 = r17
.def temp2 = r18
.def temp3 = r19
.def temp4 = r20
.def temp5 = r21
.def temp6 = r22
.def temp7 = r23
.def temp8 = r24
.def temp9 = r25

.def XL		=R26
.def XH		=R27
.def YL		=R28
.def YH		=R29
.def ZL		=R30		; ZEIGER fr TEXTE
.def ZH		=R31

.cseg					;Beginn eines Code-Segmentes
.org 0					;Startadresse=0



rjmp RESET      ; Reset Handler                           
reti ;rjmp EXT_INT0   ; IRQ0 Handler                      
reti ;rjmp EXT_INT1   ; IRQ1 Handler                      
reti ;rjmp TIM2_COMP  ; Timer2 Compare Handler            
reti ;rjmp TIM2_OVF   ; Timer2 Overflow Handler           
reti ;rjmp TIM1_CAPT  ; Timer1 Capture Handler            
reti ;rjmp TIM1_COMPA ; Timer1 CompareA Handler           
reti ;rjmp TIM1_COMPB ; Timer1 CompareB Handler           
rjmp TIM1_OVF   ; Timer1 Overflow Handler           	  
reti ;rjmp TIM0_OVF   ; Timer0 Overflow Handler           
reti ;rjmp SPI_STC    ; SPI Transfer Complete Handler     
rjmp ZEICHEN_EINLESEN ; USART RX Complete Handler     	  
reti ;rjmp USART_UDRE ; UDR Empty Handler                 
reti ;rjmp USART_TXC  ; USART TX Complete Handler         
reti ;rjmp ADC        ; ADC Conversion Complete Handler   
reti ;rjmp EE_RDY     ; EEPROM Ready Handler              
reti ;rjmp ANA_COMP   ; Analog Comparator Handler         
reti ;rjmp TWSI       ; Two-wire Serial Interface Handler 
reti ;rjmp SPM_RDY    ; Store Program Memory Ready Handler


RESET:                        
; STACK initialisieren
	ldi temp, LOW(RAMEND)      ; LOW-Byte der obersten RAM-Adresse
	out SPL, temp
	ldi temp, HIGH(RAMEND)     ; HIGH-Byte der obersten RAM-Adresse
	out SPH, temp
; PORT B initialisieren	
	ldi temp, 0b00111111    ; QUARZ / LCD 		
	out DDRB, temp
; PORT B setzen			
	out PORTB, temp

; PORT C initialisieren	
	ldi temp, 0b00001000    ; LED /Messeingnge
	out DDRC, temp
; PORT C setzen			
	out PORTC, temp

; PORT D  initialisieren	
	ldi temp, 0b11000010	; RXD/TXD/4xTASTER/2xLED
	out DDRD, temp
; PORT D setzen			
	ldi temp, 0b11111111	; PULL-UPS aktivieren 	
	out PORTD, temp
; LCD BACKLIGHT			
	cbi PORTC, LED_C_LCD	; LCD  an	

;USART initialisieren		
	ldi temp, low(BRR)  		; Baudrate 					
	out UBRRL,temp
	ldi temp,high(BRR)  		; Baudrate 					
	out UBRRH,temp
   	ldi temp, (1<<URSEL)|(3<<UCSZ0)	; Frame-Format: 8 Bit		
    out UCSRC, temp
    sbi UCSRB, RXCIE                ; Interrupt bei Empfang     
    sbi UCSRB, RXEN 				; RX (Empfang) aktivieren   
	sbi UCSRB, TXEN  				; TX aktivieren             
; Watchdog aktivieren (max. Zeit)
	ldi TEMP,(1<<WDCE)|(1<<WDE)  
	out wdtcr,TEMP       
	ldi TEMP,(1<<WDE) | (1<<WDP2)|(1<<WDP1)|(1<<WDP0) 
	out wdtcr,temp
; PAUSE					
	rcall wait_100ms
; LCD initialisieren	
	rcall lcd_init	
; ANZEIGE_TEXT_BEGRUESSUNG
	rcall ANZEIGE_TEXT_BEGRUESSUNG
; LCD_SONDERZEICHEN_PROGRAMMIEREN_0	
	rcall LCD_SONDERZEICHEN_PROGRAMMIEREN_0		; ; ""								
; LCD_SONDERZEICHEN_PROGRAMMIEREN_4	
	rcall LCD_SONDERZEICHEN_PROGRAMMIEREN_4		; /h				

; Timer1-Vorteiler 					  
; TCCR1B ==> ICNC1 ICES1 R WGM13 WGM12 CS12 CS11 CS10 
; 000 TIMER aus                 
; 001 CLOCK=TAKTFREQUENZ / 1    
; 010 CLOCK=TAKTFREQUENZ / 8    
; 011 CLOCK=TAKTFREQUENZ / 64   
; 100 CLOCK=TAKTFREQUENZ / 256  
; 101 CLOCK=TAKTFREQUENZ / 1024 
; 110 Abfallende Flanke Pin T0  
; 111 Steigende  Flanke Pin T0  
	ldi temp, 1<<CS12|0<<CS11|0<<CS10  ; 256	
	out TCCR1B,temp          
;Timer1-berlauf-Int aktivieren       
 	ldi temp,(1<<toie1)
 	out timsk,temp          

; ADC initialisieren mit "Free Run" und Vorteiler 128 	
; Bit 7 -ADEN: ADC Enable                             	
; Bit 6 -ADSC: ADC Start Conversion                   	
; Bit 5 -ADFR: ADC Free Running Select                	
; Bit 4 -ADIF: ADC Interrzpt Flag                     	
; Bit 3 -ADIE: ADC Interrupt Enable                   	
; Bit 2 -ADPS2: Vorteiler                             	
; Bit 1 -ADPS2: Vorteiler                             	
; Bit 0 -ADPS2: Vorteiler                             	
;	ADPS2 ADPS1 ADPS0 Division Factor					
;	0 		0 	0 		2								
;	0 		0 	1 		2								
;	0 		1 	0 		4								
;	0 		1 	1 		8								
;	1 		0 	0 		16								
;	1 		0 	1	 	32								
;	1 		1 	0 		64								
;	1 		1 	1 		128								
;														
	ldi temp, ((1<<ADEN)|(1<<ADSC)|(1<<ADFR)|(1<<ADPS2)|(1<<ADPS1)|(0<<ADPS0)) ; VT: 64
	out ADCSR, temp
; ADC initialisieren  ###########################################
; REFS1 REFS0 Voltage Reference Selection											
;  0     0    AREF, Internal Vref turned off										
;  0     1    AVCC with external capacitor at AREF pin								
;  1     1    Internal 2.56V Voltage Reference with external capacitor at AREF pin	
	ldi temp, 1<<REFS1 | 1<<REFS0
	out ADMUX, temp

; SRAM lschen								
SRAM_CLEAR:
	ldi ZL, LOW (adr_DATEN)
	ldi ZH, HIGH(adr_DATEN)
	clr temp
SRAM_CLEAR_w:
	ST z+,temp
	cpi ZH, HIGH(RAMEND); Vergleich?		
	brlo SRAM_CLEAR_w	; kleiner => SPRUNG	
	cpi ZL, LOW(RAMEND)	; Vergleich?		
	brlo SRAM_CLEAR_w	; kleiner => SPRUNG	
	breq SRAM_CLEAR_w	; gleich  => SPRUNG	


;											
; STARTWERTE								
;											


; REGISTER clear							
 	clr TASTE
	clr MENUE_0
	clr MENUE_1
; EXTRF + PORF FLAG zurcksetzen			
	in temp, MCUCSR
	andi temp, 0b11111100
	out MCUCSR,temp
; LEDs off									
;	sbi PORTD, LED_D_BLAU	; BLAU aus	
;	sbi PORTD, LED_D_ROT	; ROT  aus	

; FEHLERPRFUNG TASTATUR					
	rcall FEHLERPRUEFUNG_TASTATUR
; SYSTEM_EINSTELLUNGEN_EEPROM_LADEN			
	rcall SYSTEM_EINSTELLUNGEN_EEPROM_LADEN
; PROGRAMMSTARTS_ZHLEN						
;	rcall PROGRAMMSTARTS_ZAEHLEN
; REFERENZ laden							
	rcall GPS_REFERENZ_LADEN	
; GPS_DATENEMPFANG aktivieren				
	rcall GPS_CLEAR_START		

;Interrupts allgemein aktivieren			
sei                   



;######################################################################## 
;######################################################################## 
;######################################################################## 
loop:
	wdr									; WATCH-DOG-RESET	

	rcall BETRIEBSSTUNDEN_SPEICHERN		; BETRIEBSSTUNDEN 	
	rcall BERECHNUNG_ONLINE_OFFLINE		; ZEITEN ONLINE/OFFLINE
	rcall BERECHNUNG_HOEHE				; GPS Hhe Referenz		
	rcall BERECHNUNG_PEGEL				; PEGEL der Satelliten addieren
	rcall GPS_BERECHNUNG_ENTFERNUNG

	rcall wait_100ms

	rcall ABFRAGE_TASTATUR				; TASTATUR			

	rcall STEUERUNG_MENUE_EBENE_0				; MENUE
	rcall STEUERUNG_MENUE_POSITIONEN_LADEN		; MENUE
	rcall STEUERUNG_MENUE_POSITIONEN_SPEICHERN	; MENUE
	rcall STEUERUNG_MENUE_EDIT					; MENUE

rjmp loop



;###################################################################
;###################################################################
;###################################################################
FEHLERPRUEFUNG_TASTATUR:
	wdr
	rcall ABFRAGE_TASTATUR
	tst TASTE
	brne FEHLERPRUEFUNG_TASTATUR_FEHLER
	ret
FEHLERPRUEFUNG_TASTATUR_FEHLER:

	rcall ANZEIGE_TEXT_ERROR
	
	rcall wait_100ms

	rjmp FEHLERPRUEFUNG_TASTATUR


;###################################################################
;###################################################################
;###################################################################
; OUT: TASTE														
ABFRAGE_TASTATUR:
	in temp, PIND			; einlesen		
	andi temp, 0b00111100	; Bitmuster		
	com temp				; invertieren	
	andi temp, 0b00111100	; Bitmuster		
	LSR temp				; 1 x rechts	
	LSR temp				; 1 x rechts	
	cpi temp,0				; = 0 ? (keine TASTE gedrckt)	
	breq ABFRAGE_TASTATUR_0
	cpi temp,1				; = 1 ? (TASTE-1) ESC			
	breq ABFRAGE_TASTATUR_4
	cpi temp,2				; = 2 ? (TASTE-2)	-			
	breq ABFRAGE_TASTATUR_3
	cpi temp,4				; = 4 ? (TASTE-3)	+			
	breq ABFRAGE_TASTATUR_2
	cpi temp,8				; = 8 ? (TASTE-4) ENTER			
	breq ABFRAGE_TASTATUR_1
	
	ret
ABFRAGE_TASTATUR_0:
	clr TASTE				; (TASTE-0) keine TASTE		
	ret
ABFRAGE_TASTATUR_1:
 	clr TASTE				; (TASTE-1) ESC				
	clr MENUE_0
	clr MENUE_1
	rcall LCD_CLEAR
	rcall wait_100ms
	ret
ABFRAGE_TASTATUR_2: 
	ldi temp,TASTE_MINUS	; (TASTE-2)	MINUS			
	mov TASTE,temp
	rcall wait_100ms
	rcall LCD_CLEAR
	ret
ABFRAGE_TASTATUR_3:
	ldi temp,TASTE_PLUS		; (TASTE-3)	PLUS			
	mov TASTE,temp
	rcall wait_100ms
	rcall LCD_CLEAR
	ret
ABFRAGE_TASTATUR_4:
	ldi temp,TASTE_ENTER	; (TASTE-4) ENTER			
	mov TASTE,temp
	rcall wait_100ms
	rcall LCD_CLEAR
ret
;######################################################################## 
;######################################################################## 
;######################################################################## 	
;																		  	
; rcall ABFRAGE_TASTATUR_TASTE_ENTER ;OUT: FLAGS (bei ENTER-TASTE=>TASTE=0)	
;																		  	
ABFRAGE_TASTATUR_TASTE_ENTER:
	mov temp, TASTE
	cpi temp, TASTE_ENTER
	brne ABFRAGE_TASTATUR_TASTE_ENTER_w
	; TASTE clear	
	clr TASTE
ABFRAGE_TASTATUR_TASTE_ENTER_w:
ret
;######################################################################## 
;######################################################################## 
;######################################################################## 
;																		  	
; rcall ABFRAGE_TASTATUR_TASTE_MINUS ; OUT: FLAGS							
;																		  	
ABFRAGE_TASTATUR_TASTE_MINUS:
	mov temp, TASTE
	cpi temp, TASTE_MINUS
ret
;######################################################################## 
;######################################################################## 
;######################################################################## 
;																		  	
; rcall ABFRAGE_TASTATUR_TASTE_PLUS ; OUT: FLAGS							
;																		  	
ABFRAGE_TASTATUR_TASTE_PLUS:
	mov temp, TASTE
	cpi temp, TASTE_PLUS
ret


;#######################################################################
;#######################################################################
;#######################################################################
;																		
; 	INPUT: temp1														
;																		
;	rcall SENDEN 		; INP: temp1									
;																		
SENDEN:
	sbis UCSRA,UDRE     	; Warten bis UDR frei             	
	rjmp senden				; Sprung							
	out UDR, temp1			; senden							
ret

;######################################################################## 
;######################################################################## 
;######################################################################## 
.include "GPS_LCD.asm"
.include "GPS_ANZEIGEN.asm" 
.include "GPS_FUNKTIONEN.asm" 
.include "GPS_INTERRUPT.asm" 
.include "GPS_BERECHNUNG.asm"
.include "GPS_TEXTE.asm"
.include "GPS_MENU.asm"
.include "GPS_EEPROM.asm"
